/*

*/

import renderer.scene.*;
import renderer.models.Disk;
import renderer.models.ModelShading;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.awt.Color;
import java.awt.event.*;

/**
   This example demonstrates a form of "letterboxing".
<p>
   This program uses Java's resize window event handler and
   the FrameBuffer's viewport to solve the problem of changing
   aspect ratios demonstrated by the program Circle_v1.java
<p>
   This program always uses a square viewport (which matches the
   shape of the camera's view rectangle), which is as large as
   possible and is always positioned in the lower left hand corner
   of this program's FrameBuffer.
<p>
   As an exercise, modify this program to always place the
   square viewport in the upper left hand corner of the
   FrameBuffer.
<p>
   Also try placing the square viewport centered on the left
   edge of the FrameBuffer.
<p>
   Finally, try placing the square viewport at the center of
   the FrameBuffer (true letterboxing).
*/
public class Circle_v2_Letterboxing
{
   private static final int SIZE = 512;

   private static double left   = -1.0;
   private static double right  =  1.0;
   private static double bottom = -1.0;
   private static double top    =  1.0;

   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'c' key to change the random solid model color.");
      System.out.println("Use the 'C' key to randomly change model's colors.");
      System.out.println("Use the 'e' key to change the random solid edge colors.");
      System.out.println("Use the 'E' key to change the random edge colors.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }


   public static void main(String[] args)
   {
      print_help_message();

      // Create a model of a circle.
      Position model_p = new Position( new Disk(1.0, 10, 40) );

      ModelShading.setRandomColor(model_p.model);

      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addPosition(model_p);

      // Set up the camera's view volume.
      scene.camera.projOrtho(left, right, bottom, top);


      // Create a InteractiveFrame and a FrameBuffer to render our scene into.
      int width  = SIZE;
      int height = SIZE;

      @SuppressWarnings("serial")
      final InteractiveFrame app = new InteractiveFrame("Renderer 7: Letterboxing", width, height)
      {
         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );

            char c = e.getKeyChar();
            if ('h' == c)
            {
               print_help_message();
               return;
            }
            else if ('d' == c)
            {
               Pipeline.debug = ! Pipeline.debug;
             //RasterizeAntialias.debug = ! RasterizeAntialias.debug;
            }
            else if ('a' == c)
            {
               RasterizeAntialias.doAntialiasing = ! RasterizeAntialias.doAntialiasing;
               System.out.print("Anti-aliasing is turned ");
               System.out.println(RasterizeAntialias.doAntialiasing ? "On" : "Off");
            }
            else if ('c' == c)
            {
               // Change the solid random color of the disk.
               ModelShading.setRandomColor(scene.positionList.get(0).model);
            }
            else if ('C' == c)
            {
               // Change each color in the disk to a random color.
               ModelShading.setRandomColors(scene.positionList.get(0).model);
            }
            else if ('e' == c && e.isAltDown())
            {
               // Change the random color of each vertex of the disk.
               ModelShading.setRandomVertexColors(scene.positionList.get(0).model);
            }
            else if ('e' == c)
            {
               // Change the solid random color of each edge of the disk.
               ModelShading.setRandomLineSegmentColors(scene.positionList.get(0).model);
            }
            else if ('E' == c)
            {
               // Change the random color of each end of each edge of the disk.
               ModelShading.setRainbowLineSegmentColors(scene.positionList.get(0).model);
            }

            // Render again.
            this.getFrameBuffer().clearFB(Color.black);
            Pipeline.render(scene, this.getFrameBuffer().vp);
            fbp.update();
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
            System.out.println( e );

            // Get the new size of the FrameBufferPanel.
            int w = this.fbp.getWidth();
            int h = this.fbp.getHeight();

            // Create a new FrameBuffer that fits the new window size.
            FrameBuffer fb = new FrameBuffer(w, h);
            this.fbp.setFrameBuffer(fb);
            fb.clearFB(Color.darkGray);

            System.out.printf("Aspect ratio (of framebuffer) = %.2f\n", (double)w/(double)h);

            // Create a viewport, with aspect ratio 1, in the
            // lower left hand corner of the FrameBuffer.
            int d = Math.min(w, h);
            fb.setViewport(0, h - d, d, d);
            fb.vp.clearVP(Color.black);

            Pipeline.render(scene, fb.vp);
            fbp.update();
            repaint();
         }
      };
   }
}
