/*

*/

import renderer.scene.*;
import renderer.models.Disk;
import renderer.models.ModelShading;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.awt.Color;
import java.awt.event.*;

/**
  The example demonstrates a form of "cropping".
<p>
  This program keeps the aspect ratio and the size of the complete
  circle at a constant 512 by 512 pixels. Also, the complete circle
  is anchored in the upper left hand corner of this program's window.
  But the program window can be any size. If the program window is
  smaller than 512 by 512 pixels, then only part of the circle is
  shown in the program window (the view of our scene gets "cropped"
  to fit inside of the screen window).
<p>
  Run this program and resize the window so that it is smaller than
  512 by 512 and see what happens.
<p>
  To create this effect, we call from within Java's componentResized()
  event handler the Camera's projOrtho() method. As the user changes
  the shape (the aspect ratio) and size of the screen window, we need
  to change the shape and size of the camera's view rectangle to always
  match the screen window.
<p>
  As an exercise, modify this program to always have the complete circle
  anchored in the lower right hand corner of this program's window. Try
  resizing the window using either the window's lower right hand corner
  or the window's upper left hand corner.
<p>
  As an exercise, modify this program to always have the complete circle
  centered at the center of the screen window (that is, centered in the
  FrameBuffer).
*/
public class Circle_v3a_Cropping
{
   private static final int SIZE = 512;

   private static double left   = -1.0;
   private static double right  =  1.0;
   private static double bottom = -1.0;
   private static double top    =  1.0;

   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'c' key to change the random solid model color.");
      System.out.println("Use the 'C' key to randomly change model's colors.");
      System.out.println("Use the 'e' key to change the random solid edge colors.");
      System.out.println("Use the 'E' key to change the random edge colors.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }


   public static void main(String[] args)
   {
      print_help_message();

      // Create four models of a circle.
      Position model1_p = new Position( new Disk(1.0, 10, 40) );
      Position model2_p = new Position( new Disk(1.0, 10, 40) );
      Position model3_p = new Position( new Disk(1.0, 10, 40) );
      Position model4_p = new Position( new Disk(1.0, 10, 40) );

      ModelShading.setRandomColor(model1_p.model);
      ModelShading.setRandomColor(model2_p.model);
      ModelShading.setRandomColor(model3_p.model);
      ModelShading.setRandomColor(model4_p.model);

      // Translate three of the models.
      model2_p.matrixMult( Matrix.translate(2,  0, 0) );
      model3_p.matrixMult( Matrix.translate(0, -2, 0) );
      model4_p.matrixMult( Matrix.translate(2, -2, 0) );

      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Add the Models to the Scene.
      scene.addPosition(model1_p, model2_p, model3_p, model4_p);

      // Set up the camera's view volume.
      scene.camera.projOrtho(left, right, bottom, top);


      // Create a InteractiveFrame and a FrameBuffer to render our scene into.
      int width  = SIZE;
      int height = SIZE;

      @SuppressWarnings("serial")
      final InteractiveFrame app = new InteractiveFrame("Renderer 7: Cropping", width, height)
      {
         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );

            char c = e.getKeyChar();
            if ('h' == c)
            {
               print_help_message();
               return;
            }
            else if ('d' == c)
            {
               Pipeline.debug = ! Pipeline.debug;
             //RasterizeAntialias.debug = ! RasterizeAntialias.debug;
            }
            else if ('a' == c)
            {
               RasterizeAntialias.doAntialiasing = ! RasterizeAntialias.doAntialiasing;
               System.out.print("Anti-aliasing is turned ");
               System.out.println(RasterizeAntialias.doAntialiasing ? "On" : "Off");
            }
            else if ('c' == c)
            {
               // Change the solid random color of the disk.
               ModelShading.setRandomColor(scene.positionList.get(0).model);
            }
            else if ('C' == c)
            {
               // Change each color in the disk to a random color.
               ModelShading.setRandomColors(scene.positionList.get(0).model);
            }
            else if ('e' == c && e.isAltDown())
            {
               // Change the random color of each vertex of the disk.
               ModelShading.setRandomVertexColors(scene.positionList.get(0).model);
            }
            else if ('e' == c)
            {
               // Change the solid random color of each edge of the disk.
               ModelShading.setRandomLineSegmentColors(scene.positionList.get(0).model);
            }
            else if ('E' == c)
            {
               // Change the random color of each end of each edge of the disk.
               ModelShading.setRainbowLineSegmentColors(scene.positionList.get(0).model);
            }

            // Render again.
            this.getFrameBuffer().clearFB(Color.black);
            Pipeline.render(scene, this.getFrameBuffer().vp);
            fbp.update();
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
            System.out.println( e );

            // Get the new size of the FrameBufferPanel.
            int w = this.fbp.getWidth();
            int h = this.fbp.getHeight();

            // Create a new FrameBuffer that fits the new window size.
            FrameBuffer fb = new FrameBuffer(w, h);
            this.fbp.setFrameBuffer(fb);
            fb.clearFB(Color.black);

            System.out.printf("Aspect ratio (of framebuffer) = %.2f\n", (double)w/(double)h);

            // Create a view volume whose aspect ratio matches the
            // FrameBuffer's (and the viewport's) aspect ratio.
            left  = -1;
            right = -1 + 2.0 * (w / (double)SIZE);
            bottom = 1 - 2.0 * (h / (double)SIZE);
            top    = 1;
            scene.camera.projOrtho(left, right, bottom, top);

            Pipeline.render(scene, fb.vp);
            fbp.update();
            repaint();
         }
      };
   }
}
