/*

*/

import renderer.scene.*;
import renderer.models.Disk;
import renderer.models.ModelShading;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.awt.Color;
import java.awt.event.*;

/**
  The example demonstrates a form of "pan-and-scan".
<p>
  This program keeps the aspect ratio and the size of the complete
  circle at a constant 512 by 512 pixels. The program window can be
  any size. If the program window is smaller than 512 by 512 pixels,
  then only part of the complete circle is shown in the program window
  (the view of our circle in the camera's view rectangle gets "cropped"
  to fit inside of the screen window).
*/
public class Circle_v5_Pan_and_Scan
{
   private static final int SIZE = 512;

   private static int sliderValueH = 50;
   private static int sliderValueV = 50;

   private static double left   = -1.0;
   private static double right  =  1.0;
   private static double bottom = -1.0;
   private static double top    =  1.0;

   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'c' key to change the random solid model color.");
      System.out.println("Use the 'C' key to randomly change model's colors.");
      System.out.println("Use the 'e' key to change the random solid edge colors.");
      System.out.println("Use the 'E' key to change the random edge colors.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }


   public static void main(String[] args)
   {
      print_help_message();

      // Create a model of a circle.
      Position model_p = new Position( new Disk(1.0, 10, 40) );

      ModelShading.setRandomColor(model_p.model);

      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addPosition(model_p);

      // Set up the camera's view volume.
      scene.camera.projOrtho(left, right, bottom, top);


      // Create a InteractiveFrame and a FrameBuffer to render our scene into.
      int width  = SIZE;
      int height = SIZE;

      @SuppressWarnings("serial")
      final InteractiveFrame app = new InteractiveFrame_HV("Renderer 7: Pan and Scan", width, height)
      {
         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );

            char c = e.getKeyChar();
            if ('h' == c)
            {
               print_help_message();
               return;
            }
            else if ('d' == c)
            {
               Pipeline.debug = ! Pipeline.debug;
             //RasterizeAntialias.debug = ! RasterizeAntialias.debug;
            }
            else if ('a' == c)
            {
               RasterizeAntialias.doAntialiasing = ! RasterizeAntialias.doAntialiasing;
               System.out.print("Anti-aliasing is turned ");
               System.out.println(RasterizeAntialias.doAntialiasing ? "On" : "Off");
            }
            else if ('c' == c)
            {
               // Change the solid random color of the disk.
               ModelShading.setRandomColor(scene.positionList.get(0).model);
            }
            else if ('C' == c)
            {
               // Change each color in the disk to a random color.
               ModelShading.setRandomColors(scene.positionList.get(0).model);
            }
            else if ('e' == c && e.isAltDown())
            {
               // Change the random color of each vertex of the disk.
               ModelShading.setRandomVertexColors(scene.positionList.get(0).model);
            }
            else if ('e' == c)
            {
               // Change the solid random color of each edge of the disk.
               ModelShading.setRandomLineSegmentColors(scene.positionList.get(0).model);
            }
            else if ('E' == c)
            {
               // Change the random color of each end of each edge of the disk.
               ModelShading.setRainbowLineSegmentColors(scene.positionList.get(0).model);
            }

            // Render again.
            this.getFrameBuffer().clearFB(Color.black);
            Pipeline.render(scene, this.getFrameBuffer().vp);
            fbp.update();
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
            System.out.println( e );

            // Get the new size of the FrameBufferPanel.
            int w = this.fbp.getWidth();
            int h = this.fbp.getHeight();

            // Create a new FrameBuffer that fits the new window size.
            FrameBuffer fb = new FrameBuffer(w, h);
            this.fbp.setFrameBuffer(fb);

            System.out.printf("Aspect ratio (of framebuffer) = %.2f\n", (double)w/(double)h);

            draw();
         }

         // Implement the AdjustmentListener interface for the scrollbar.
         @Override public void adjustmentValueChanged(AdjustmentEvent e)
         {
          //super.adjustmentValueChanged(e);
            sliderValueV = this.sbV.getValue();
            sliderValueH = this.sbH.getValue();
            System.out.println("slider_H = " + sliderValueH + ", "
                             + "slider_V = " + sliderValueV);
            draw();
         }

         // Render the current Scene into the FrameBuffer's viewport.
         void draw()
         {
            // Get the size of the FrameBuffer.
            FrameBuffer fb = this.fbp.getFrameBuffer();
            int w = fb.width;
            int h = fb.height;

            // Create a viewport.
            int vpX = 0;
            int vpY = 0;
            int vpW = w;
            int vpH = h;
            if (w > SIZE)
            {
               vpW = SIZE;
               vpX = (w - SIZE)/2;
            }
            if (h > SIZE)
            {
               vpH = SIZE;
               vpY = (h - SIZE)/2;
            }
            fb.setViewport(vpX, vpY, vpW, vpH);
            fb.clearFB(Color.darkGray);
            fb.vp.clearVP(Color.black);

            // Create a view volume whose aspect ratio matches the
            // viewport's aspect ratio.
            left   = -(vpW/(double)SIZE);
            right  =  (vpW/(double)SIZE);
            bottom = -(vpH/(double)SIZE);
            top    =  (vpH/(double)SIZE);
            // Use the slider value to pan the view volume across the scene.
            left   += (sliderValueH - 50.0)/50.0 * (1.0 - vpW/(double)SIZE);
            right  += (sliderValueH - 50.0)/50.0 * (1.0 - vpW/(double)SIZE);
            bottom += (50.0 - sliderValueV)/50.0 * (1.0 - vpH/(double)SIZE);
            top    += (50.0 - sliderValueV)/50.0 * (1.0 - vpH/(double)SIZE);
            scene.camera.projOrtho(left, right, bottom, top);

            Pipeline.render(scene, fb.vp);
            fbp.update();
            repaint();
         }
      };
   }
}
